angular.module('configAppliances', [])

    .controller('AppliancesCtrl', ['$scope', '$rootScope', '$http', function($scope, $rootScope, $http) {

        $scope.initCtrl = function () {
            processAppliances(PHD, window);
        };

        $scope.listViewFlag =false;
        $scope.tableView = true;
        $scope.getView = function(tabular) {
            var view = tabular || $scope.tableView;
            return (view  ===true ? "Table" : "List");
        };

        $scope.STORAGE_TAB = 0;
        $scope.TARGETS_TAB = 1;
        $scope.NETWORK_TAB = 2;
        $scope.ASSETS_TAB = 3;

        $scope.toggleView = function() {
            $scope.tableView = !$scope.tableView;
            
            if($scope.currentDetail == $scope.STORAGE_TAB){
                $scope.setApplianceStorageTab();
            } if ($scope.currentDetail == $scope.TARGETS_TAB) {
                $scope.setApplianceTargetsTab();
            } else {
                $scope.setApplianceNetworkTab();
            }

            if(!$scope.tableView){
                $scope.selectApplianceFromList($scope.selectedID);
                $("#" + ($scope.selectedID)).css("background-color","#f5f5f5");
            }
        };

        function processAppliances(PHD, window) {
            var     REFRESH_INTERVAL = 60000;
            var     STORAGE_TAB = $scope.STORAGE_TAB,
                    TARGETS_TAB = $scope.TARGETS_TAB,
                    NETWORK_TAB = $scope.NETWORK_TAB,
                    ASSETS_TAB = $scope.ASSETS_TAB;

            var storageTypeLookup = {
                "backup": gettext("Backup"),
                "archive": gettext("Archive"),
                "sync": gettext("Sync")
            };

            var storageDialogTitle = gettext("Add %(storage_type)s %(usage)s Storage");

            var backupCopyDialogTitle = gettext("Add Backup Copy %(storage_type)s Target");

            function usageLookup(usage) {
                var icon = '<i class="icon-uui-backup' + ' icon-large text-standard" title="' + 'Backup' + '"></i> ';
                if (usage == 'archive') {
                    icon = '<i class="icon-uui-backup-copy icon-large text-standard" title="' + 'Backup Copy' + '">' +
                        '<span class="path1"></span><span class="path2"></span><span class="path3"></span>' +
                        '<span class="path4"></span>' +
                        '<span class="path5"></span><span class="path6"></span><span class="path7"></span></i> ';
                }
                return icon;
            }

            var storage_icon_map = {};
            storage_icon_map[""] = "harddrive";
            storage_icon_map["internal"] = "harddrive";
            storage_icon_map["stateless"] = "harddrive";
            storage_icon_map["disk"] = "harddrive";
            storage_icon_map["iscsi"] = "harddrive";
            storage_icon_map["added_internal"] = "harddrive";

            function getStorageType(row) {
                var type = 'attached';
                var icon_name = 'harddrive';
                if (row.type === 'nas') {
                    if (row.properties !== undefined && row.properties != null) {
                        icon_name = type = row.properties.protocol;
                    }
                } else {
                    icon_name = storage_icon_map[row.type];
                }
                var icon = '<i class="icon-uui-' + icon_name + ' icon-large text-standard" title="' + type + '"></i>';
                return icon;
            }

            var cellFormatter = {
                role: function(value, row) {
                    var value = row.role;
                    var html = title = "";
                    if (typeof value !== 'undefined') {
                        if (value[2]) {
                            title = gettext("Presentation Appliance");
                            html += '<i class="icon-phd-layer-presentation icon-large vba-pres" title="' + title + '"></i>';
                        }
                        if (value[1]) {
                            title = gettext("Management Appliance");
                            html += '<i class="icon-phd-layer-management icon-large vba-mgmt" title="' + title + '"></i>';
                        }
                        if (value[0]) {
                            title = gettext("Engine Appliance");
                            html += '<i class="icon-phd-layer-engine icon-large vba-eng" title="' + title + '"></i>';
                        }
                    }
                    return html;
                },
                replication: function(value, row) {
                    var role = row.role;
                    // use role to see if replication is true or false.
                    return "Not Enabled";
                },
                total_mb_size: function(value, row) {
                    var size = row.total_mb_size;
                    var free = row.total_mb_free;
                    var used = size / free;
                    var used_allocated = "";
                    var percent_used = 0;
                    var state = "storage-";
                    var warn_threshold = 0.75;
                    var stop_threshold = 0.95;
                    if(used < warn_threshold) state += "ok";
                    if(used >= warn_threshold && used < stop_threshold) state += "warn";
                    if(used >= stop_threshold) state += "critical";
                    if (row.total_mb_size !== undefined) {
                        var sizeVal = row.total_mb_size;
                        var freeVal = row.total_mb_free;
                        if (sizeVal === 0) {
                            used_allocated = "Not Available";
                            color = "color:gray";
                            percent_used = 100;
                        } else {
                            var size = PHD.util.formatMB(sizeVal);
                            var used = PHD.util.formatMB(sizeVal - freeVal);
                            used_allocated = "" + used + " / " + size;
                            percent_used = ((sizeVal - freeVal) * 100.0) / sizeVal;
                        }
                    }
                    return '<div class="progress-bar" title="' + used_allocated + '">' +
                        '<div class="progress-meter ' + state + '" style="width:' + percent_used + '%;"></div></div>';
                },
                type: function (value, row) {
                    var usageIcon = usageLookup(row.usage),
                        stoIcon = getStorageType(row);
                    return usageIcon + stoIcon;
                },
                status: function(value, row) {
                    var displayStatus = "Online";
                    var iconClass = "okcircle";
                    if (value === 'pending') {
                        iconClass = "restart icon-spin";
                        displayStatus = "Pending";
                    } else if (value === "offline") {
                        iconClass = "exclamationsign";
                        displayStatus = "Offline";
                    } else if (value === 'deleting') {
                        iconClass = "restart icon-spin";
                        displayStatus = "Deleting";
                    }
                    return '<i class="icon-'+iconClass+' icon-large" title="' + displayStatus + '"></i> ' + displayStatus;
                },
                version: function(value, row) {
                    var icon = title = "";
                    //if(row.state.version_info === VERSION_NOT_UP_TO_DATE) {
                    if(row.version_status === "update_available") {
                        title = gettext("An update is available. Use the Global tab to apply updates.");
                        icon = '<i class="icon-phd-update-needed" title="'+title+'"></i>';
                        //} else if (row.state.version_info === VERSION_UPDATE_IN_PROGRESS) {
                    } else if (row.version_info === VERSION_UPDATE_IN_PROGRESS) {
                        title = gettext("Update in progress");
                        icon = '<i class="icon-phd-update-in-progress icon-pulse" title="'+title+'"></i>';
                    }
                    return icon + " " + value;
                },
                mb_size: function(value, row) {
                    return PHD.util.formatMB(value);
                },
                mb_free: function(value, row) {
                    return PHD.util.formatMB(value);
                },
                max_size: function(value, row) {
                    if (value != 0) {
                        return PHD.util.formatMB(value * 1024);
                    } else {
                        return value;
                    }
                },
                percent_used: function (value, row) {
                    var percent_used = row.percent_used,
                        state = "storage-";
                    var used_allocated = "";
                    var color = "";
                    if (row.mb_size !== undefined) {
                        var sizeVal = row.mb_size;
                        var freeVal = row.mb_free;
                        if (sizeVal === 0) {
                            used_allocated = "Not Available";
                            percent_used = "0%";
                        } else {
                            var size = PHD.util.formatMB(sizeVal);
                            var free = PHD.util.formatMB(freeVal);
                            used_allocated = "" + free + " / " + size;
                        }
                    }
                    var will_purge = row.mb_to_purge;
                    var to_display = used_allocated;
                    return '<div class="progress-bar" title="' + to_display + '">' +
                        '<div class="progress-meter ' + state + 'ok" style="width:' + percent_used + ';' + color + '"></div></div>';
                }
            };

            var networkFormatter  = {
                dns: function(value) {
                    var html = "";
                    if(value[0]) {
                        html += '<span class="truncates item-name" title="' + value[0] + '">' + value[0] + '</span>';
                    }
                    if(value[1]) {
                        html += '<span class="truncates item-name" title="' + value[1] + '">' + value[1] + '</span>';
                    }
                    return html;
                },
                search: function(value) {
                    var html = "";
                    if(value[0]) {
                        html += '<span class="truncates item-name" title="' + value[0] + '">' + value[0] + '</span>';
                    }
                    if(value[1]) {
                        html += '<span class="truncates item-name" title="' + value[1] + '">' + value[1] + '</span>';
                    }
                    return html;
                }
            };

            var Appliance = function($el) {
                $scope.el = $el;
                this.$el = $el;
                this.dbid = this.$el.data("dbid");
                this.$header = $el.children(".environment-header");
                this.$appliances = $el.children(".environment-appliances");

                this.$buttons = $el.find(".button-bar a");
                $scope.buttons = this.$buttons;
                this.$options = this.$buttons.filter(".btn-environment-options");

                this.appliancesInit = false;

                var applianceOptions = {
                    cellClasses: ["long-text truncate", null, null, null, null, "numerical", null, null]
                };

                this.$applianceTable = this.$appliances.find(".appliance-table"); //this.$appliances.find(".appliance-table");
                this.ApplianceTable = PHD.DataTable(this.$applianceTable, applianceOptions).registerFormatters(cellFormatter);
                $scope.ApplianceTable = this.ApplianceTable;

                this.appliances = [];
                this.$selectedAppliance = $();
                
                this.selectedID = null;
                $scope.selectedID = null;
                PHD.appliance_sid = this.selectedID;


                this.$storageTable = this.$el.find(".bds-table");
                this.StorageTable = PHD.DataTable(this.$storageTable, {}).registerFormatters(cellFormatter);
                $scope.StorageTalbeObj = this.StorageTable;
                
                this.$storageTableList = this.$el.find(".bds-table-list");
                this.StorageTableList = PHD.DataTable(this.$storageTableList, {}).registerFormatters(cellFormatter);
                $scope.StorageTableListObj = this.StorageTableList;

                this.storageData = [];

                this.$targetsTable = this.$el.find(".targets-table");
                this.TargetsTable = PHD.DataTable(this.$targetsTable, {}).registerFormatters(cellFormatter);
                $scope.TargetsTableObj = this.TargetsTable;

                this.$targetsTableList = this.$el.find(".targets-table-list");
                this.TargetsTableList = PHD.DataTable(this.$targetsTableList, {}).registerFormatters(cellFormatter);
                $scope.TargetsTableListObj = this.TargetsTableList;

                this.targetsData = [];

                this.$networkTable = this.$el.find(".network-table");
                this.NetworkTable = PHD.DataTable(this.$networkTable, {integerIDs: false}).registerFormatters(networkFormatter);
                $scope.NetworkTableObj = this.NetworkTable;
                
                this.$networkTableList = this.$el.find(".network-table-list");
                this.NetworkTableList = PHD.DataTable(this.$networkTableList, {integerIDs: false}).registerFormatters(networkFormatter);
                $scope.NetworkTableListObj = this.NetworkTableList;

                this.currentDetail = STORAGE_TAB;
                $scope.currentDetail = this.currentDetail;

                this.$selectedStorage = $();
                this.selectedStorageID = null;

                this.$selectedTarget = $();
                this.selectedTargetID = null;

                this.$selectedNic = $();
                this.selectedNicID = null;

                this.applianceProcess = null;
                this.detailProcess = null;

                this.timestamp = 0;
                this.refreshInterval = null;
                this.isActive = false;
            };
           
            Appliance.prototype.init = function() {
                var self = this;

                this.initButtons();

                this.$applianceTable
                    .on("rowselect", function(event, dbid, $el, DataTable) {
                        self.$selectedAppliance = $el;
                        $scope.el = $el;
                        self.selectedID = dbid;
                        PHD.appliance_sid = self.selectedID;
                        $scope.selectedID = self.selectedID;
                        self.$selectedStorage = $();
                        self.selectedStorageID = null;
                        self.$selectedTarget = $();
                        self.selectedTargetID = null;
                        self.$selectedNic = $();
                        self.selectedNicID = null;
                        self.updateButtons();
                    
                        if($(".storagesubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                            this.currentDetail = STORAGE_TAB;
                        }else if($(".targetssubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                            this.currentDetail = TARGETS_TAB;
                        }else if($(".networksubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                            this.currentDetail = NETWORK_TAB;
                        }else{
                            this.currentDetail = $scope.currentDetail;
                        }
                    
                        self.refresh(this.updateAppliances, false);
                    })
                    .on("loadfinished updatefinished", function(event, newRows, DataTable) {
                        self.$applianceTable.scrollTable({"height": ".environment-appliances"});
                        if(event.type === "loadfinished" && self.appliances.length) {
                            //self.ApplianceTable.selectByID(self.appliances[0].id);
                        }
                    });

                this.$storageTable
                    .on("click", ".help-trigger i", function(event) {
                        var $anchor = $(this).parent();

                        $("<div>", {
                            html: $anchor.data("value")
                        }).popover({
                            autoOpen: true,
                            anchor: $anchor,
                            popoverClass: "nowrap",
                            position: {
                                my: "center bottom",
                                at: "center top-5"
                            }
                        });
                    })
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedStorage = $el;
                        self.selectedStorageID = dbid;
                        self.updateButtons();
                    })
                    .on("loadfinished updatefinished", function(event, newRows, DataTable) {
                        self.updateButtons();
                    });

                this.$storageTableList
                    .on("click", ".help-trigger i", function(event) {
                        var $anchor = $(this).parent();

                        $("<div>", {
                            html: $anchor.data("value")
                        }).popover({
                            autoOpen: true,
                            anchor: $anchor,
                            popoverClass: "nowrap",
                            position: {
                                my: "center bottom",
                                at: "center top-5"
                            }
                        });
                    })
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedStorage = $el;
                        self.selectedStorageID = dbid;
                        self.updateButtons();
                    })
                    .on("loadfinished updatefinished", function(event, newRows, DataTable) {
                        self.updateButtons();
                    });


                this.$targetsTable
                    .on("click", ".help-trigger i", function(event) {
                        var $anchor = $(this).parent();

                        $("<div>", {
                            html: $anchor.data("value")
                        }).popover({
                            autoOpen: true,
                            anchor: $anchor,
                            popoverClass: "nowrap",
                            position: {
                                my: "center bottom",
                                at: "center top-5"
                            }
                        });
                    })
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedTarget = $el;
                        self.selectedTargetID = dbid;
                        self.updateButtons();
                    })
                    .on("loadfinished updatefinished", function(event, newRows, DataTable) {
                        self.updateButtons();
                    });

                this.$targetsTableList
                    .on("click", ".help-trigger i", function(event) {
                        var $anchor = $(this).parent();

                        $("<div>", {
                            html: $anchor.data("value")
                        }).popover({
                            autoOpen: true,
                            anchor: $anchor,
                            popoverClass: "nowrap",
                            position: {
                                my: "center bottom",
                                at: "center top-5"
                            }
                        });
                    })
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedTarget = $el;
                        self.selectedTargetID = dbid;
                        self.$selectedStorage = $el;
                        self.selectedStorageID = dbid;
                        self.updateButtons();
                    })
                    .on("loadfinished updatefinished", function(event, newRows, DataTable) {
                        self.updateButtons();
                    });
                

                this.$networkTable
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedNic = $el;
                        self.selectedNicID = dbid;
                        self.updateButtons();
                    });
                
                this.$networkTableList
                    .on("rowselect", ".clickable", function(event, dbid, $el, DataTable) {
                        self.$selectedNic = $el;
                        self.selectedNicID = dbid;
                        self.updateButtons();
                    });

                $(document)
                    .on("bdssave nicsave", function(event, applianceID, data) {
                        if(self.selectedID === applianceID)
                            self.fetchDetails(true);
                    })
                    .on("vbaadd", function(event) {
                        self.addVBA();
                    });

                this.$appliances.find(".sub-tabs-wrapper").tabs({
                    activate: function(event, ui) {
                        self.currentDetail = ui.newTab.index();
                        $scope.currentDetail = self.currentDetail;
                        console.log('currentDetail is ' + self.currentDetail + ' and isactive = ' + self.isActive);
                        
                        if(self.selectedID && self.isActive) {
                            self.fetchDetails(true);
                        }
                    }
                });

                return this;
            };

           
            
            Appliance.prototype.initButtons = function() {
                var self = this;

                this.$el.find(".environment-options-menu").flyout();
                this.$el.find(".vba-storage-menu").flyout();

                this.$el
                    .on("click", ".btn-add-attached", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedID) {
                            var bds = $(this).closest(".menu-group").data("bds"),
                                title = interpolate(storageDialogTitle, {
                                    storage_type: gettext("Attached Disk"),
                                    usage: storageTypeLookup[bds]
                                }, true);

                            selectedStorage = null;

                            PHD.wizard('app/configure/storage/attached/attached.html', "add-bds", {
                                helpArticle: PHD.App.getHelpLink("addbds", true),
                                title: title,
                                width: 1024,
                                height: 700,
                                open: function(event, ui) {
                                    $("#id_usage").val(bds);
                                    $("#id_type").val(5);
                                }
                            });
                        }
                    })
                    .on("click", ".btn-add-cifs", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedID) {
                            var bds = $(this).closest(".menu-group").data("bds"),
                                title = interpolate(storageDialogTitle, {
                                    storage_type: gettext("CIFS"),
                                    usage: storageTypeLookup[bds]
                                }, true);

                            selectedStorage = null;

                            // lmc - build page by loading template and variable value. differentiate usage?
                            PHD.wizard('app/configure/storage/cifs/cifs.html', "add-bds", {
                                helpArticle: PHD.App.getHelpLink("addbds", true),
                                title: title,
                                width: 500,
                                height: 480,
                                open: function(event, ui) {
                                    $("#id_usage").val(bds);
                                }
                            });
                        }
                    })
                    .on("click", ".btn-add-nfs", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedID) {
                            var bds = $(this).closest(".menu-group").data("bds"),
                                title = interpolate(storageDialogTitle, {
                                    storage_type: gettext("NFS"),
                                    usage: storageTypeLookup[bds]
                                }, true);

                            //PHD.wizard('/config/bds/'+bds.toLowerCase()+'/nfs/' + self.selectedID + "/", "add-bds", {
                            // lmc - build page by loading template and variable value. differentiate types?
                            PHD.wizard('app/configure/storage/nfs/nfs.html', "add-bds", {
                                helpArticle: PHD.App.getHelpLink("addbds", true),
                                title: title,
                                width: 500,
                                height: 480,
                                open: function(event, ui) {
                                    $("#id_usage").val(bds);
                                }
                            });
                        }
                    })
                    .on("click", ".btn-add-ircache", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedID) {
                            //PHD.wizard('/config/bds/ir_cache/attached/' + self.selectedID + "/", "add-bds", {
                            // lmc - build page by loading template and variable value. differentiate types?
                            PHD.wizard('app/configure/storage/attached/ir.html', "add-bds", {
                                helpArticle: PHD.App.getHelpLink("addbds", true),
                                title: gettext("Add Instant Recovery Write Space"),
                                helpArticle: PHD.App.getHelpLink("addircache", true),
                                width: 1024,
                                height: 520,
                                open: function(event, ui) {
                                    $("#id_usage").val(6); // TODO - usage for "IR"
                                    $("#id_type").val(5); // added internal disk only.
                                }
                            });
                        }
                    })
                    .on("click", ".btn-add-cloud", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedID) {
                            var bds = $(this).closest(".menu-group").data("bds"),
                                title = interpolate(backupCopyDialogTitle, {
                                    storage_type: gettext("Cloud")
                                }, true);

                            selectedStorage = null;

                            // lmc - build page by loading template and variable value. differentiate usage?
                            PHD.wizard('app/configure/storage/cloud/cloud.html', "add-bds", {
                                helpArticle: PHD.App.getHelpLink("addcloud", true),
                                title: title,
                                width: 900,
                                height: 500,
                                open: function(event, ui) {
                                    $("#id_usage").val(bds);
                                }
                            });
                        }
                    })
                    .on("click", ".btn-edit-storage", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedStorageID) {
                            console.log('selected id = ' + self.selectedStorageID);
                            var storage = _.find(self.storageData, function(obj) { return obj.id == self.selectedStorageID });
                            if (storage !== undefined) {
                                var type = storage.type;
                                var properties = storage.properties;
                                var protocol = properties !== undefined ? properties.protocol : null;
                                console.log('type = ' + type);
                                var spec = getHTMLSpec(type, protocol);
                                var message;

                                var url = 'app/configure/storage/' + spec.htmlFile;

                                selectedStorage = storage;
                                if (selectedStorage.name == 'Internal' || selectedStorage.usage == 'stateless') {
                                    var resp = PHD.Ajax.get("/api/storage/ir");
                                    resp.done(function(data) {
                                        var ir_GB = data["ir storage"].allowed;
                                        var d2d_GB;
                                        PHD.wizard(url, "edit-storage", {
                                            helpArticle: PHD.App.getHelpLink("addbds", true),
                                            title: gettext("Edit Storage"),
                                            width: spec.width,
                                            height: spec.height,
                                            open: function(event, ui) {
                                                $("body")
                                                    .addClass("dialog-open")
                                                    .css("overflow", "auto");
                                                $("#id_name").val(storage.name);
                                                $("#id_name").disable();
                                                $("#edit_object_id").text(storage.id);
                                                $("#id_type").disable(); // type is not changeable on edit.
                                                $("#id_usage").disable(); // usage is not changeable on edit
                                                if (properties !== undefined) {
                                                    $("#id_host").val(properties.hostname);
                                                    $("#id_share").val(properties.share_name);
                                                    $("#id_username").val(properties.username);
                                                    $("#id_port").val(properties.port);
                                                }
                                                var storage_GB = Math.round(selectedStorage.mb_size / 1024);
                                                ir_GB = parseInt(ir_GB, 10);
                                                d2d_GB = storage_GB - ir_GB;
                                                var ir_percent = Math.round(ir_GB / storage_GB * 100);
                                                var d2d_percent = 100 - ir_percent;
                                                $("#id_ir_gb").text("(" + ir_GB + " GB)");
                                                $("#id_d2d_gb").text("(" + d2d_GB + " GB)");
                                                $("#id_ir_allocation").val(ir_percent);
                                                $("#id_d2d_allocation").val(d2d_percent);
                                            }
                                        });
                                    });
                                } else {
                                    PHD.wizard(url, "edit-storage", {
                                        helpArticle: PHD.App.getHelpLink("addbds", true),
                                        title: gettext("Edit Storage"),
                                        width: spec.width,
                                        height: spec.height,
                                        open: function(event, ui) {
                                            $("body")
                                                .addClass("dialog-open")
                                                .css("overflow", "auto");
                                            $("#id_name").val(storage.name);
                                            $("#id_name").disable();
                                            $("#edit_object_id").text(storage.id);
                                            //$("#id_type").disable(); // type is not changeable on edit.
                                            $("#id_usage").disable(); // usage is not changeable on edit
                                            if (properties !== undefined) {
                                                $("#id_host").val(properties.hostname);
                                                $("#id_share").val(properties.share_name);
                                                $("#id_username").val(properties.username);
                                                $("#id_port").val(properties.port);
                                            }
                                        }
                                    });
                                }
                            } else {
                                message = "Error finding this storage, please level and re-enter the configuration subsystem and retry.";
                                PHD.throwError({"result": [{"message": gettext(message)}]});
                            }
                        }
                    })
                    .on("click", ".btn-edit-target", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedTargetID) {
                            console.log('selected id = ' + self.selectedTargetID);
                            var target = _.find(self.targetsData, function(obj) { return obj.id == self.selectedTargetID });
                            if (target !== undefined) {
                                var type = target.type;
                                var properties = target.properties;
                                var protocol = properties !== undefined ? properties.protocol : null;
                                console.log('type = ' + type);
                                var spec = getHTMLSpec(type, protocol);
                                var message;

                                var url = 'app/configure/storage/' + spec.htmlFile;

                                selectedStorage = target;
                                PHD.wizard(url, "edit-target", {
                                    helpArticle: PHD.App.getHelpLink("addbds", true),
                                    title: gettext("Edit Backup Copy Target"),
                                    width: spec.width,
                                    height: spec.height,
                                    open: function(event, ui) {
                                        $("body")
                                            .addClass("dialog-open")
                                            .css("overflow", "auto");
                                        $("#id_name").val(target.name);
                                        $("#id_name").disable();
                                        $("#edit_object_id").text(target.id);
                                        //$("#id_type").disable(); // type is not changeable on edit.
                                        $("#id_usage").disable(); // usage is not changeable on edit
                                        if (properties !== undefined) {
                                            $("#id_host").val(properties.hostname);
                                            $("#id_share").val(properties.share_name);
                                            $("#id_username").val(properties.username);
                                            $("#id_port").val(properties.port);
                                            var max_size = properties.max_size;
                                            if (max_size !== undefined) {
                                                $("#id_max_size").val(max_size == 0 ? 2048 : max_size);
                                            }
                                        }
                                    }
                                });
                            } else {
                                message = "Error finding this target, please level and re-enter the configuration subsystem and retry.";
                                PHD.throwError({"result": [{"message": gettext(message)}]});
                            }
                        }
                    })
                    .on("click", ".btn-onoff-storage", function(event) {
                        if(self.selectedStorageID) {
                            var storage = _.find(self.storageData, function(obj) { return obj.id == self.selectedStorageID });
                            if (storage !== undefined) {
                                if (storage.name === "Internal" && storage.status === 'online') {
                                    PHD.throwError({"result": [{"message": gettext("Internal storage cannot be disabled.")}]});
                                } else if (storage.status === 'deleting') {
                                    PHD.throwError({"result": [{"message": gettext("Storage being deleted cannot be enabled.")}]});
                                } else {
                                    var dbid = self.selectedStorageID;
                                    var storage = _.findWhere(self.storageData, {"id": dbid});
                                    var url = '/api/storage/';
                                    if (storage) {
                                        if (storage.status === 'online') {
                                            url += 'offline' + '/' + dbid + '/?sid=' + self.selectedID;
                                        } else if (storage.status === 'offline') {
                                            url += 'online' + '/' + dbid + '/?sid=' + self.selectedID;
                                        }
                                    }
                                    var appliance = self;
                                    var resp = PHD.Ajax.put(url);
                                    resp.done(function (data) {
                                        appliance.fetchDetails(true);
                                    });
                                }
                            }
                        }
                    })
                    .on("click", ".btn-onoff-target", function(event) {
                        if(self.selectedTargetID) {
                            var target = _.find(self.targetsData, function(obj) { return obj.id == self.selectedTargetID });
                            if (target !== undefined) {
                                if (target.status === 'deleting') {
                                    PHD.throwError({"result": [{"message": gettext("Target being deleted cannot be enabled.")}]});
                                } else {
                                    var dbid = self.selectedTargetID;
                                    var target = _.findWhere(self.targetsData, {"id": dbid});
                                    var url = '/api/storage/';
                                    if (target) {
                                        if (target.status === 'online') {
                                            url += 'offline' + '/' + dbid + '/?sid=' + self.selectedID;
                                        } else if (target.status === 'offline') {
                                            url += 'online' + '/' + dbid + '/?sid=' + self.selectedID;
                                        }
                                    }
                                    var appliance = self;
                                    var resp = PHD.Ajax.put(url);
                                    resp.done(function (data) {
                                        appliance.fetchDetails(true);
                                    });
                                }
                            }
                        }
                    })
                    .on("click", ".btn-remove-storage", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedStorageID) {
                            var storage = _.find(self.storageData, function(obj) { return obj.id == self.selectedStorageID });
                            if (storage !== undefined) {
                                if (storage.name === "Internal") {
                                    PHD.throwError({"result": [{"message": gettext("Internal storage cannot be deleted.")}]});
                                } else if (storage.status === 'deleting') {
                                    PHD.throwError({"result": [{"message": gettext("This storage is already being deleted.")}]});
                                } else {
                                    var dbid = self.selectedStorageID,
                                        confirm = PHD.confirmDialog(
                                            "<p>"+gettext("Are you sure you want to remove this storage?")+"</p>" +
                                            "<p>"+gettext("Removing unmounts the storage volume from the appliance. Attached disks remain connected to the appliance virtual machine and can be reused or deleted manually.")+"</p>",
                                            { title: gettext("Confirm Storage Removal") }
                                        );

                                    confirm.done(function() {
                                        var url = '/api/storage/' + self.selectedStorageID + '/?sid=' + self.selectedID;
                                        var resp = PHD.Ajax.delete(url);
                                        resp.done(function(data) {
                                            self.removeBDS(data, dbid);
                                        });
                                    });
                                }
                            } else {
                                message = "Error finding this storage, please level and re-enter the configuration subsystem and retry.";
                                PHD.throwError({"result": [{"message": gettext(message)}]});
                            }
                        }
                    })
                    .on("click", ".btn-remove-target", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedTargetID) {
                            var target = _.find(self.targetsData, function(obj) { return obj.id == self.selectedTargetID });
                            if (target !== undefined) {
                                if (target.status === 'deleting') {
                                    PHD.throwError({"result": [{"message": gettext("This target is already being deleted.")}]});
                                } else {
                                    var dbid = self.selectedTargetID,
                                        confirm = PHD.confirmDialog(
                                            "<p>"+gettext("Are you sure you want to remove this target?")+"</p>" +
                                            "<p>"+gettext("Removing a target does not delete backups that have been copied to it.  ") +
                                            gettext("If you wish to remove the backups, you should use the \"Erase\" button.") + "</p>",
                                            { title: gettext("Confirm Target Removal") }
                                        );

                                    confirm.done(function() {
                                        var url = '/api/storage/' + self.selectedTargetID + '/?sid=' + self.selectedID;
                                        var resp = PHD.Ajax.delete(url);
                                        resp.done(function(data) {
                                            self.removeTarget(data, dbid);
                                        });
                                    });
                                }
                            } else {
                                message = "Error finding this target, please level and re-enter the configuration subsystem and retry.";
                                PHD.throwError({"result": [{"message": gettext(message)}]});
                            }
                        }
                    })
                    .on("click", ".btn-prepare-target", function(event) {
                        if(self.selectedTargetID) {
                            var dbid = self.selectedTargetID;
                            var target = _.findWhere(self.targetsData, {"id": dbid});
                            if (target !== undefined) {
                                if (target.properties != undefined && target.properties.protocol == 'cloud') {

                                    confirm = PHD.confirmDialog(
                                        "<p>"+gettext("Are you sure you want to erase the target ") + target.name + "?" + "</p>" +
                                        "<p>"+gettext("Erasing a target will delete all backups that have been copied to it.")+"</p>",
                                        { title: gettext("Confirm Erase") }
                                    );

                                    confirm.done(function() {

                                        var sid = "/?sid=" + self.selectedID;
                                        var appliance = self;
                                        var check_url = '/api/archive/media/connected' + sid;
                                        var load = PHD.showLoadingIndicator(appliance.$targetsTable, "Scanning Targets...");
                                        var resp = PHD.Ajax.get(check_url, load);
                                        resp.done(function (data) {
                                            if (data.data !== undefined) {

                                                var url = '/api/archive/media/prepare/';
                                                if (target) {
                                                    url += target.name + '/?sid=' + self.selectedID;
                                                }
                                                var obj = {};
                                                obj.label = "Cloud_Target";
                                                obj = JSON.stringify(obj);
                                                var load = PHD.showLoadingIndicator(appliance.$storageTable, "Preparing Target...");
                                                var resp = PHD.Ajax.put(url, obj, load);
                                                resp.done(function (data) {
                                                    appliance.fetchDetails(true);
                                                });
                                            }
                                        });
                                    });
                                } else {
                                    PHD.throwError({"result": [{"message": gettext("Only cloud targets can be completely erased.")}]});
                                }
                            }
                        }
                    })
                    .on("click", ".btn-edit-nic", function(event) {
                        event.preventDefault();
                        if(!$(this).data("disabled") && self.selectedNicID) {
                            PHD.wizard("app/configure/networks/edit.html", "edit-nic", {
                                title: gettext("Edit Network Adapter ") + self.selectedNicID,
                                width: 500,
                                height: 500,
                                open: function(event, ui) {
                                    $("body")
                                        .addClass("dialog-open")
                                        .css("overflow", "auto");
                                    var mac = self.$selectedNic.find("[data-source=\"mac\"]").html();
                                    var ip = self.$selectedNic.find("[data-source=\"ip\"]").html();
                                    var gateway = self.$selectedNic.find("[data-source=\"gateway\"]").html();
                                    var netmask = self.$selectedNic.find("[data-source=\"netmask\"]").html();
                                    var dns1 = self.$selectedNic.find("[data-source=\"dns1\"]").html();
                                    var dns2 = self.$selectedNic.find("[data-source=\"dns2\"]").html();
                                    var network_type = self.$selectedNic.find("[data-source=\"network_type\"]").html();
                                    $("h4").text("MAC Address: " + mac);
                                    $("#id_ip").val(ip);
                                    $("#id_gateway").val(gateway);
                                    $("#id_subnet").val(netmask);
                                    $("#id_dns1").val(dns1);
                                    $("#id_dns2").val(dns2);
                                    $("#edit_object_id").text(self.selectedNicID);
                                }
                            });
                        }
                    })
                    .on("click", ".btn-refresh-nics", function(event) {
                        event.preventDefault();
                        if(self.selectedID) {
                            self.$selectedNic = $();
                            self.selectedNicID = null;

                            var resp = PHD.Ajax.get('/api/networks/?sid=' + self.selectedID);
                            resp.done(function(data){
                                self.buildNetwork(data);
                            });
                        }
                    })
                    .on("click", ".btn-add-appliance", function(event) {
                        event.preventDefault();
                        PHD.wizard("app/configure/appliances/add.html", "add-vba", {
                            title: gettext("Add Appliance"),
                            width: 600,
                            height: "auto",
                            helpArticle: PHD.App.getHelpLink("addvba", true)
                        });
                    })
                    .on("click", ".btn-edit-appliance", function(event) {
                        event.preventDefault();
                        selectedAppliance = _.find(self.appliances, function(obj) { return obj.id == self.selectedID });
                        PHD.wizard("app/configure/appliances/edit.html", "edit-vba", {
                            title: gettext("Edit Appliance"),
                            width: 600,
                            height: "auto",
                            helpArticle: PHD.App.getHelpLink("addvba", true),
                            open: function(event, ui) {
                                $("body")
                                    .addClass("dialog-open")
                                    .css("overflow", "auto");
                            },
                            beforeClose: function (event, ui) {
                               self.refresh(self.fetchAppliances);
                            },
                            close: function(event,ui){
                                $("body")
                                    .removeClass("dialog-open")
                                    .css("overflow", "hidden");
                                $(this).wizard("destroy").remove();
                            }
                        });
                    })
                    .on("click", ".btn-remove-appliance", function() {
                        var confirm = PHD.confirmDialog(
                            "<p>"+gettext("Are you sure you want to remove the appliance?")+"</p>",
                            { title: gettext("Confirm Appliance Removal") }
                        );

                        confirm.done($.proxy(self.kill, self));
                    });

                return this;
            };

            Appliance.prototype.setRefreshInterval = function(interval) {
                interval = interval || REFRESH_INTERVAL;
                clearInterval(this.refreshInterval);
                
                if($(".storagesubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                    this.currentDetail = STORAGE_TAB;
                }else if($(".targetssubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                    this.currentDetail = TARGETS_TAB;
                }else if($(".networksubtab").closest("li").hasClass("ui-tabs-active ui-state-active")){
                    this.currentDetail = NETWORK_TAB;
                }else{
                    this.currentDetail = $scope.currentDetail;
                }
                
                this.refreshInterval = setTimeout($.proxy(this.refresh, this), interval);
                return this;
            };

            Appliance.prototype.refresh = function(fn, updateDetails) {
                var self = this;

                if (!$rootScope.maintenance_mode) {
                    if (this.isActive || $scope.listViewFlag) {
                        fn = fn || this.updateAppliances;
                        updateDetails = !(updateDetails === false);

                        this.applianceProcess = this.fetchAppliances(fn);
                        this.detailProcess = this.fetchDetails(updateDetails);

                        $.when(this.applianceProcess, this.detailProcess).done(function () {
                            self.applianceProcess = null;
                            self.detailProcess = null;
                            if (!self.selectedID && self.appliances.length) self.ApplianceTable.selectByID(self.appliances[0].id);
                            self.setRefreshInterval();
                        });
                    }
                } else {
                    console.log("In maintenance mode: Skipping appliance refresh.....")
                }
            };

            Appliance.prototype.fetchAppliances = function(fn) {
                if(!_.isFunction(fn)) {
                    fn = this.buildAppliances;
                }
                //this.ApplianceTable.showLoadingIndicator();

                return PHD.Ajax
                    //lmc
                    .get("/api/systems/?timestamp=" + this.timestamp)
                    .done($.proxy(fn, this));
            };

            Appliance.prototype.fetchDetails = function(update) {
                var fn;
                this.selectedID = $scope.selectedID;
                
                update = (update === true);

                if (!this.selectedID) {
                    return $.Deferred().resolve();
                }

                if(this.currentDetail === STORAGE_TAB) {
                    if (update === true) {
                        fn = this.updateBDS;
                    } else {
                        fn = this.buildBDS;
                    }
                    console.log('in detail storage, id is ' + this.selectedID);
                    return PHD.Ajax
                        .get("/api/storage/?sid=" + this.selectedID + "&usage=backup")
                        .done($.proxy(fn, this))
                        .always($.proxy(this.hideLoaders, this));
                } else if (this.currentDetail == TARGETS_TAB) {
                    if (update === true) {
                        fn = this.updateTargets;
                    } else {
                        fn = this.buildTargets;
                    }
                    console.log('in detail targets, id is ' + this.selectedID);
                    return PHD.Ajax
                        .get("/api/storage/?sid=" + this.selectedID + "&usage=archive")
                        .done($.proxy(fn, this))
                        .always($.proxy(this.hideLoaders, this));
                } else if(this.currentDetail === NETWORK_TAB) {
                    console.log('in detail network, id is ' + this.selectedID);
                    if(update === true) {
                        //fn = this.updateNetwork; TODO - API does not return sid.
                        fn = this.buildNetwork;
                    } else {
                        fn = this.buildNetwork;
                    }

                    return PHD.Ajax
                        .get("/api/networks/?sid=" + this.selectedID)
                        .done($.proxy(fn, this))
                        .always($.proxy(this.hideLoaders, this));
                }
            };

              
            $scope.applianceDataArr = [];
            $scope.selectedAppliance = $();
            Appliance.prototype.buildAppliances = function(data) {
                if(data) {
                    this.appliances = data.appliance;
                    this.timestamp = data.timestamp;
                    this.ApplianceTable.load(this.appliances);
                    
                    $scope.applianceDataArr = data.appliance;
                    $scope.selectedAppliance = $scope.applianceDataArr[0];
                    $("#" + ($scope.applianceDataArr[0].id)).css("background-color","#f5f5f5");
                }
                if(this.applianceProcess) this.applianceProcess.resolve();
            };

            Appliance.prototype.updateAppliances = function(data) {
                if(data) {
                    this.appliances = data.appliance;
                    this.timestamp = data.timestamp;
                    //this.ApplianceTable.update(this.appliances);
                }
                if(this.applianceProcess) this.applianceProcess.resolve();
            };

            Appliance.prototype.buildBDS = function(data) {
                if (data && data.storage !== undefined) {
                    this.storageData = data.storage;
                    
                    if(this.StorageTable == undefined){
                        this.StorageTable = $scope.StorageTalbeObj;
                    }

                    if(this.StorageTableList == undefined){
                        this.StorageTableList = $scope.StorageTableListObj;
                    }
                   
                    this.StorageTable.load(this.storageData);
                    this.StorageTableList.load(this.storageData);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };

            Appliance.prototype.updateBDS = function(data) {
                if(data && data.storage != undefined) {
                    this.storageData = data.storage;

                    if(this.StorageTable == undefined){
                        this.StorageTable = $scope.StorageTalbeObj;
                    }

                    if(this.StorageTableList == undefined){
                         this.StorageTableList = $scope.StorageTableListObj;
                    }

                    this.StorageTable.update(this.storageData, true);
                    this.StorageTableList.update(this.storageData, true);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };

            Appliance.prototype.buildTargets = function(data) {
                if (data && data.storage !== undefined) {
                    if (data.storage.length > 0) {
                        data.storage.forEach(function(obj, index) {
                            obj.max_size = obj.properties.max_size !== undefined ? obj.properties.max_size : 0;
                            obj.hostname = obj.properties.hostname !== undefined ? obj.properties.hostname : "";
                        });
                    }
                    this.targetsData = data.storage;

                    if(this.TargetsTable == undefined){
                        this.TargetsTable = $scope.TargetsTableObj;
                    }

                    if(this.TargetsTableList == undefined){
                        this.TargetsTableList = $scope.TargetsTableObj;
                    }

                    this.TargetsTable.load(this.targetsData);
                    this.TargetsTableList.load(this.targetsData);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };

            Appliance.prototype.updateTargets = function(data) {
                if(data && data.storage != undefined) {
                    if (data.storage.length > 0) {
                        data.storage.forEach(function(obj, index) {
                            obj.max_size = obj.properties.max_size !== undefined ? obj.properties.max_size : 0;
                            obj.hostname = obj.properties.hostname !== undefined ? obj.properties.hostname : "";
                        });
                    }
                    this.targetsData = data.storage;

                    if(this.TargetsTable == undefined){
                        this.TargetsTable = $scope.TargetsTableObj;
                    }

                    if(this.TargetsTableList == undefined){
                        this.TargetsTableList = $scope.TargetsTableObj;
                    }

                    this.TargetsTable.update(this.targetsData, true);
                    this.TargetsTableList.update(this.targetsData, true);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };

            Appliance.prototype.removeBDS = function(data, dbid) {
                this.StorageTableList.deselect();
                this.StorageTable.deselect();
                this.$selectedStorage = $();
                this.selectedStorageID = null;
                this.fetchDetails(true);
                this.updateButtons();
            };

            Appliance.prototype.removeTarget = function(data, dbid) {
                this.TargetsTableList.deselect();
                this.TargetsTable.deselect();
                this.$selectedTarget = $();
                this.selectedTargetID = null;
                this.fetchDetails(true);
                this.updateButtons();
            };

            Appliance.prototype.buildNetwork = function(data) {
                //if(data && data.id === this.selectedID) {
                // lmc - know result is of specified system
                if (data) {
                    this.networkdata = data.networks;

                    if(this.NetworkTable == undefined){
                        this.NetworkTable = $scope.NetworkTableObj;
                    }

                    if(this.NetworkTableList == undefined){
                         this.NetworkTableList = $scope.NetworkTableListObj;
                    }
                    
                    this.NetworkTable.load(this.networkdata);
                    this.NetworkTableList.load(this.networkdata);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };

            Appliance.prototype.updateNetwork = function(data) {
                if(data && data.id === this.selectedID) {
                    this.networkdata = data.networks;

                    if(this.NetworkTable == undefined){
                        this.NetworkTable = $scope.NetworkTableObj;
                    }

                    if(this.NetworkTableList == undefined){
                         this.NetworkTableList = $scope.NetworkTableListObj;
                    }

                    this.NetworkTable.update(this.networkdata, true);
                    this.NetworkTableList.update(this.networkdata, true);
                    this.updateButtons();
                }
                if(this.detailProcess) this.detailProcess.resolve();
            };

            Appliance.prototype.isReady = function() {
                return this.isActive && (this.applianceProcess === null || this.applianceProcess.state() === 'resolved') &&
                    (this.detailProcess === null || this.detailProcess.state() === 'resolved');
            };

            Appliance.prototype.updateButtons = function() {
                if(this.$buttons == undefined){
                    this.$buttons = $scope.buttons;
                }
                
                this.$buttons.filter(".btn-edit-storage")
                    .toggleButton(this.selectedID && this.selectedStorageID);
                this.$buttons.filter(".btn-remove-storage")
                    .toggleButton(this.selectedID && this.selectedStorageID);
                this.$buttons.filter(".btn-edit-target")
                    .toggleButton(this.selectedID && this.selectedTargetID);
                this.$buttons.filter(".btn-remove-target")
                    .toggleButton(this.selectedID && this.selectedTargetID);
                this.$buttons.filter(".btn-edit-nic")
                    .toggleButton(this.selectedID && this.selectedNicID);

                // check for storage status.
                $enableDisable = this.$buttons.filter(".btn-onoff-storage");
                if (this.selectedStorageID) {
                    var dbid = this.selectedStorageID;
                    var storage = _.findWhere(this.storageData, {"id": dbid});
                    if (storage != null && storage.status === 'online') {
                        $enableDisable.html('<i class="icon-off icon-large"></i> Disable');
                    } else {
                        $enableDisable.html('<i class="icon-restart icon-large"></i> Enable');
                    }
                }

                // check for target status.
                $enableDisable = this.$buttons.filter(".btn-onoff-target");
                if (this.selectedTargetID) {
                    var dbid = this.selectedTargetID;
                    var target = _.findWhere(this.targetsData, {"id": dbid});
                    if (target != null && target.status === 'online') {
                        $enableDisable.html('<i class="icon-off icon-large"></i> Disable');
                    } else {
                        $enableDisable.html('<i class="icon-restart icon-large"></i> Enable');
                    }
                }
                return this;
            };

            Appliance.prototype.setButtonLock = function(locked) {
                this.$buttons.each(function() {
                    $(this).toggleButton(!locked);
                });
            };

            Appliance.prototype.initAppliances = function() {
                this.refresh(this.buildAppliances);
                this.appliancesInit = true;
                return this;
            };

            Appliance.prototype.addVBA = function() {
                this.refresh(this.updateAppliances);
            };

            Appliance.prototype.hideLoaders = function() {
                if(this.currentDetail === STORAGE_TAB) {
                    //this.StorageTable.hideLoadingIndicator();
                } else if(this.currentDetail === NETWORK_TAB) {
                    //this.NetworkTable.hideLoadingIndicator();
                }
            };

            Appliance.prototype.kill = function() {
                var self = this,
                    resp = PHD.Ajax.delete("/api/systems/" + this.selectedID);

                resp.done(function(data) {
                    console.log("in done with DELETE!");
                    //$(document).trigger("systemremove", [self.selectedID]);
                    self.selectedID = null;
                    PHD.appliance_sid = self.selectedID;
                    self.refresh(self.fetchAppliances);
                    /*if(data.result[0].code == AJAX_RESULT_SUCCESS) {
                        $(document).trigger("systemremove", [self.dbid]);
                    } else {
                        PHD.throwError(data);
                    }*/
                });
            };

            Appliance.prototype.activate = function() {
                this.isActive = true;
                if(!this.appliancesInit) {
                    this.initAppliances();
                } else {
                    this.refresh(this.updateAppliances);
                }
                return this;
            };

            Appliance.prototype.deactivate = function() {
                this.isActive = false;
                clearInterval(this.refreshInterval);
                return this;
            };
            
            var ApplianceTab = function(index, $panel) {
                this.index = index;
                this.$panel = $panel;
                this.$applianceArea = this.$panel.children("#appliances-list");
                this.environmentList = {};
                this.environments = {};
                this.activeEnvironment = null;
                this.isActive = false;
                this.isBootstrapped = false;
            };

            ApplianceTab.prototype.fetchEnvironments = function() {
                this.bootstrap();
            };

            ApplianceTab.prototype.bootstrap = function(data) {

                this.$environments = this.$applianceArea.find(".environment");

                //this.environmentList = this.$environments.map(function() {
               //     return new Appliance($(this)).init();
                //}).get();

                this.environmentList = new Appliance(this.$environments).init();
                this.activeEnvironment = this.environmentList.activate();
                this.isBootstrapped = true;
            };

            ApplianceTab.prototype.init = function() {
                var self = this;
                $(document)
                    .on("systemremove", function(event, dbid) {

                        //self.removeEnvironment(dbid);
                    });

                return this;
            };

            ApplianceTab.prototype.activate = function() {
                this.isActive = true;
                if(!this.isBootstrapped) {
                    this.fetchEnvironments();
                }
                if(this.activeEnvironment) {
                    this.activeEnvironment.activate();
                }
                return this;
            };

            ApplianceTab.prototype.deactivate = function() {
                this.isActive = false;
                if(this.activeEnvironment) {
                    this.activeEnvironment.deactivate();
                }
                return this;
            };

          
            
            function getHTMLSpec(type, protocol) {
                var spec = {};
                spec.width = 1024;
                spec.height = 700;
                spec.htmlFile = 'attached/attached.html';

                if (type == 'added_disk') {
                    ;
                } else if (type == 'nas') {
                    if (protocol == 'cifs') {
                        spec.htmlFile = 'cifs/cifs.html';
                        spec.width = 500;
                        spec.height = 480;
                    } else if (protocol == 'nfs') {
                        spec.htmlFile = 'nfs/nfs.html';
                        spec.width = 500;
                        spec.height = 480;
                    } else {
                        spec.htmlFile = 'cloud/cloud.html';
                        spec.width = 900;
                        spec.height = 500;
                    }
                } else if (type == 'ir') {
                    ; // TODO - need to specify IR storage path here.
                    spec.height = 520;
                }

                return spec;
            }
            
        
            Appliance.prototype.setSelectedAppliance = function(selectedId){
                var self = this;
                $scope.listViewFlag = true;
                
                self.ApplianceTable = $scope.ApplianceTable;
                
                self.currentDetail = $scope.currentDetail;
                self.$selectedAppliance = $scope.el;
                self.$selectedStorage = $();
                self.selectedStorageID = null;
                self.$selectedNic = $();
                self.selectedNicID = null;
                
                self.selectedID = selectedId;
                PHD.appliance_sid = selectedId;
                
                self.refresh(this.updateAppliances, false);
               // self.updateButtons();

                var x = this.ApplianceTable.$table[0].rows;
                for (var i = 1; i < x.length; i++) {
                    
                    var rowDetails = x[i];
                    
                    if((parseInt(x[i].dataset.dbid)) == selectedId){
                        if(!($(x[i]).hasClass("current-row"))){
                            $(x[i]).addClass("current-row");
                            $scope.selectedID = selectedId;
                            this.$selectedAppliance = $(x[i]);
                            this.selectedID = selectedId;
                        }
                        
                    }else{
                        if($(x[i]).hasClass("clickable current-row")){
                            $(x[i]).removeClass("current-row");
                        }
                    }
                }
                
                this.ApplianceTable.$table[0].rows = x;
                self.updateButtons();
            }
            
            
            $scope.selectApplianceFromList = function(selectedId){
                var arrLength=($scope.applianceDataArr).length;

                for (var i=0; i < arrLength; i++){
                    if ($scope.applianceDataArr[i].id===selectedId){
                       $scope.selectedAppliance = $scope.applianceDataArr[i];
                       $("#" + selectedId).css("background-color","#f5f5f5");
                       $scope.selectedID = selectedId;
                    }else{
                        $("#" + $scope.applianceDataArr[i].id).css("background-color","#fff");
                    }
                }
                
                Appliance.prototype.setSelectedAppliance(selectedId);
            };
            
            
            $(".storagesubtab").closest("li").click(function(){
                this.currentDetail = STORAGE_TAB;
                $scope.setApplianceStorageTab();
                Appliance.prototype.setSelectedAppliance($scope.selectedID);
            });

            $(".targetssubtab").closest("li").click(function(){
                this.currentDetail = TARGETS_TAB;
                $scope.setApplianceTargetsTab();
                Appliance.prototype.setSelectedAppliance($scope.selectedID);
            });
            
            $(".networksubtab").closest("li").click(function(){
                this.currentDetail = NETWORK_TAB;
                $scope.setApplianceNetworkTab();
                Appliance.prototype.setSelectedAppliance($scope.selectedID);
            });
            
            $scope.setApplianceStorageTab = function(){
                $scope.currentDetail = STORAGE_TAB;
                $(".storagesubtab").closest("li").addClass("ui-tabs-active ui-state-active");
                $(".targetssubtab").closest("li").removeClass("ui-tabs-active ui-state-active");
                $(".networksubtab").closest("li").removeClass("ui-tabs-active ui-state-active");
                
                $("#tableStorageTab").closest("li").attr("aria-selected", true);
                $("#listStorageTab").closest("li").attr("aria-selected", true);
                $("#tableStorageTab").closest("li").attr("aria-expanded", true);
                $("#listStorageTab").closest("li").attr("aria-expanded", true);

                $("#tableTargetsTab").closest("li").attr("aria-selected", false);
                $("#listTargetsTab").closest("li").attr("aria-selected", false);
                $("#tableTargetsTab").closest("li").attr("aria-expanded", false);
                $("#listTargetsTab").closest("li").attr("aria-expanded", false);

                $("#tableNetworkTab").closest("li").attr("aria-selected", false);
                $("#listNetworkTab").closest("li").attr("aria-selected", false);
                $("#tableNetworkTab").closest("li").attr("aria-expanded", false);
                $("#listNetworkTab").closest("li").attr("aria-expanded", false);
                
                $(".config-storage").attr("aria-hidden", false);
                $(".config-targets").attr("aria-hidden", true);
                $(".config-network").attr("aria-hidden", true);
                
                $(".config-storage").css("display","block");
                $(".config-targets").css("display","none");
                $(".config-network").css("display","none");
            };
            
            $scope.setApplianceTargetsTab = function(){
                $scope.currentDetail = TARGETS_TAB;
                $(".storagesubtab").closest("li").removeClass("ui-tabs-active ui-state-active");
                $(".targetssubtab").closest("li").addClass("ui-tabs-active ui-state-active");
                $(".networksubtab").closest("li").removeClass("ui-tabs-active ui-state-active");
                
                $("#tableStorageTab").closest("li").attr("aria-selected", false);
                $("#listStorageTab").closest("li").attr("aria-selected", false);
                $("#tableStorageTab").closest("li").attr("aria-expanded", false);
                $("#listStorageTab").closest("li").attr("aria-expanded", false);

                $("#tableTargetsTab").closest("li").attr("aria-selected", true);
                $("#listTargetsTab").closest("li").attr("aria-selected", true);
                $("#tableTargetsTab").closest("li").attr("aria-expanded", true);
                $("#listTargetsTab").closest("li").attr("aria-expanded", true);

                $("#tableNetworkTab").closest("li").attr("aria-selected", false);
                $("#listNetworkTab").closest("li").attr("aria-selected", false);
                $("#tableNetworkTab").closest("li").attr("aria-expanded", false);
                $("#listNetworkTab").closest("li").attr("aria-expanded", false);

                $(".config-storage").attr("aria-hidden", true);
                $(".config-targets").attr("aria-hidden", false);
                $(".config-network").attr("aria-hidden", true);
                
                $(".config-storage").css("display","none");
                $(".config-targets").css("display","block");
                $(".config-network").css("display","none");
            };

            $scope.setApplianceNetworkTab = function(){
                $scope.currentDetail = NETWORK_TAB;
                $(".storagesubtab").closest("li").removeClass("ui-tabs-active ui-state-active");
                $(".targetssubtab").closest("li").addClass("ui-tabs-active ui-state-active");
                $(".networksubtab").closest("li").addClass("ui-tabs-active ui-state-active");

                $("#tableStorageTab").closest("li").attr("aria-selected", false);
                $("#listStorageTab").closest("li").attr("aria-selected", false);
                $("#tableStorageTab").closest("li").attr("aria-expanded", false);
                $("#listStorageTab").closest("li").attr("aria-expanded", false);

                $("#tableTargetsTab").closest("li").attr("aria-selected", false);
                $("#listTargetsTab").closest("li").attr("aria-selected", false);
                $("#tableTargetsTab").closest("li").attr("aria-expanded", false);
                $("#listTargetsTab").closest("li").attr("aria-expanded", false);

                $("#tableNetworkTab").closest("li").attr("aria-selected", true);
                $("#listNetworkTab").closest("li").attr("aria-selected", true);
                $("#tableNetworkTab").closest("li").attr("aria-expanded", true);
                $("#listNetworkTab").closest("li").attr("aria-expanded", true);

                $(".config-storage").attr("aria-hidden", true);
                $(".config-targets").attr("aria-hidden", true);
                $(".config-network").attr("aria-hidden", false);

                $(".config-storage").css("display","none");
                $(".config-targets").css("display","none");
                $(".config-network").css("display","block");
            };
            
            PHD.ApplianceTab = ApplianceTab;
        }
        
    }]);